@@ -94,6 +94,7 @@ enum DndDragType {
 };
 
 static const GtkTargetEntry drag_types_dest[] = {
+	{ "text/path-list",  0, DND_DRAG_TYPE_URI_LIST },
 	{ "text/uri-list",   0, DND_DRAG_TYPE_URI_LIST },
 	{ "text/contact-id", 0, DND_DRAG_TYPE_CONTACT_ID },
 	{ "text/plain",      0, DND_DRAG_TYPE_STRING },
@@ -101,6 +102,7 @@ static const GtkTargetEntry drag_types_dest[] = {
 };
 
 static const GtkTargetEntry drag_types_dest_file[] = {
+	{ "text/path-list",  0, DND_DRAG_TYPE_URI_LIST },
 	{ "text/uri-list",   0, DND_DRAG_TYPE_URI_LIST },
 };
 
@@ -238,6 +240,21 @@ contact_list_view_drag_got_contact (EmpathyTpContactFactory *factory,
 		data->old_group, data->new_group);
 
 	list = empathy_contact_list_store_get_list_iface (priv->store);
+
+	if (!tp_strdiff (data->new_group, EMPATHY_CONTACT_LIST_STORE_FAVORITE)) {
+		/* Mark contact as favourite */
+		empathy_contact_list_add_to_favourites (list, contact);
+		return;
+	}
+
+	if (!tp_strdiff (data->old_group, EMPATHY_CONTACT_LIST_STORE_FAVORITE)) {
+		/* Remove contact as favourite */
+		empathy_contact_list_remove_from_favourites (list, contact);
+		/* Don't try to remove it */
+		g_free (data->old_group);
+		data->old_group = NULL;
+	}
+
 	if (data->new_group) {
 		empathy_contact_list_add_to_group (list, contact, data->new_group);
 	}
@@ -246,6 +263,27 @@ contact_list_view_drag_got_contact (EmpathyTpContactFactory *factory,
 	}
 }
 
+static gboolean
+group_can_be_modified (const gchar *name,
+		       gboolean is_fake_group,
+		       gboolean adding)
+{
+	/* Real groups can always be modified */
+	if (!is_fake_group)
+		return TRUE;
+
+	/* The favorite fake group can be modified so users can
+	 * add/remove favorites using DnD */
+	if (!tp_strdiff (name, EMPATHY_CONTACT_LIST_STORE_FAVORITE))
+		return TRUE;
+
+	/* We can remove contacts from the 'ungrouped' fake group */
+	if (!adding && !tp_strdiff (name, EMPATHY_CONTACT_LIST_STORE_UNGROUPED))
+		return TRUE;
+
+	return FALSE;
+}
+
 static gboolean
 contact_list_view_contact_drag_received (GtkWidget         *view,
 					 GdkDragContext    *context,
@@ -266,23 +304,30 @@ contact_list_view_contact_drag_received (GtkWidget         *view,
 	gchar         *new_group = NULL;
 	gchar         *old_group = NULL;
 	gboolean       success = TRUE;
+	gboolean       new_group_is_fake, old_group_is_fake = TRUE;
 
 	priv = GET_PRIV (view);
 
 	sel_data = (const gchar *) gtk_selection_data_get_data (selection);
 	new_group = empathy_contact_list_store_get_parent_group (model,
-								 path, NULL);
+								 path, NULL, &new_group_is_fake);
+
+	if (!group_can_be_modified (new_group, new_group_is_fake, TRUE))
+		return FALSE;
 
 	/* Get source group information. */
 	if (priv->drag_row) {
 		source_path = gtk_tree_row_reference_get_path (priv->drag_row);
 		if (source_path) {
 			old_group = empathy_contact_list_store_get_parent_group (
-										 model, source_path, NULL);
+										 model, source_path, NULL, &old_group_is_fake);
 			gtk_tree_path_free (source_path);
 		}
 	}
 
+	if (!group_can_be_modified (old_group, old_group_is_fake, FALSE))
+		return FALSE;
+
 	if (!tp_strdiff (old_group, new_group)) {
 		g_free (new_group);
 		g_free (old_group);
@@ -878,6 +923,43 @@ contact_list_view_pixbuf_cell_data_func (GtkTreeViewColumn     *tree_column,
 	contact_list_view_cell_set_background (view, cell, is_group, is_active);
 }
 
+static void
+contact_list_view_group_icon_cell_data_func (GtkTreeViewColumn     *tree_column,
+					     GtkCellRenderer       *cell,
+					     GtkTreeModel          *model,
+					     GtkTreeIter           *iter,
+					     EmpathyContactListView *view)
+{
+	GdkPixbuf *pixbuf = NULL;
+	gboolean is_group;
+	gchar *name;
+
+	gtk_tree_model_get (model, iter,
+			    EMPATHY_CONTACT_LIST_STORE_COL_IS_GROUP, &is_group,
+			    EMPATHY_CONTACT_LIST_STORE_COL_NAME, &name,
+			    -1);
+
+	if (!is_group)
+		goto out;
+
+	if (tp_strdiff (name, EMPATHY_CONTACT_LIST_STORE_FAVORITE))
+		goto out;
+
+	pixbuf = empathy_pixbuf_from_icon_name ("emblem-favorite",
+		GTK_ICON_SIZE_MENU);
+
+out:
+	g_object_set (cell,
+		      "visible", pixbuf != NULL,
+		      "pixbuf", pixbuf,
+		      NULL);
+
+	if (pixbuf != NULL)
+		g_object_unref (pixbuf);
+
+	g_free (name);
+}
+
 static void
 contact_list_view_audio_call_cell_data_func (
 				       GtkTreeViewColumn      *tree_column,
@@ -1115,6 +1197,22 @@ contact_list_view_setup (EmpathyContactListView *view)
 		      "visible", FALSE,
 		      NULL);
 
+	/* Group icon */
+	cell = gtk_cell_renderer_pixbuf_new ();
+	gtk_tree_view_column_pack_start (col, cell, FALSE);
+	gtk_tree_view_column_set_cell_data_func (
+		col, cell,
+		(GtkTreeCellDataFunc) contact_list_view_group_icon_cell_data_func,
+		view, NULL);
+
+	g_object_set (cell,
+		      "xpad", 0,
+		      "ypad", 0,
+		      "visible", FALSE,
+		      "width", 16,
+		      "height", 16,
+		      NULL);
+
 	/* Name */
 	cell = empathy_cell_renderer_text_new ();
 	gtk_tree_view_column_pack_start (col, cell, TRUE);
@@ -1457,7 +1555,8 @@ empathy_contact_list_view_get_flags (EmpathyContactListView *view)
 }
 
 gchar *
-empathy_contact_list_view_get_selected_group (EmpathyContactListView *view)
+empathy_contact_list_view_get_selected_group (EmpathyContactListView *view,
+					      gboolean *is_fake_group)
 {
 	EmpathyContactListViewPriv *priv;
 	GtkTreeSelection          *selection;
@@ -1465,6 +1564,7 @@ empathy_contact_list_view_get_selected_group (EmpathyContactListView *view)
 	GtkTreeModel              *model;
 	gboolean                   is_group;
 	gchar                     *name;
+	gboolean                   fake;
 
 	g_return_val_if_fail (EMPATHY_IS_CONTACT_LIST_VIEW (view), NULL);
 
@@ -1478,6 +1578,7 @@ empathy_contact_list_view_get_selected_group (EmpathyContactListView *view)
 	gtk_tree_model_get (model, &iter,
 			    EMPATHY_CONTACT_LIST_STORE_COL_IS_GROUP, &is_group,
 			    EMPATHY_CONTACT_LIST_STORE_COL_NAME, &name,
+			    EMPATHY_CONTACT_LIST_STORE_COL_IS_FAKE_GROUP, &fake,
 			    -1);
 
 	if (!is_group) {
@@ -1485,6 +1586,9 @@ empathy_contact_list_view_get_selected_group (EmpathyContactListView *view)
 		return NULL;
 	}
 
+	if (is_fake_group != NULL)
+		*is_fake_group = fake;
+
 	return name;
 }
 
@@ -1521,7 +1625,7 @@ contact_list_view_group_remove_activate_cb (GtkMenuItem            *menuitem,
 	EmpathyContactListViewPriv *priv = GET_PRIV (view);
 	gchar                      *group;
 
-	group = empathy_contact_list_view_get_selected_group (view);
+	group = empathy_contact_list_view_get_selected_group (view, NULL);
 	if (group) {
 		gchar     *text;
 		GtkWindow *parent;
@@ -1549,6 +1653,7 @@ empathy_contact_list_view_get_group_menu (EmpathyContactListView *view)
 	GtkWidget                  *menu;
 	GtkWidget                  *item;
 	GtkWidget                  *image;
+	gboolean                   is_fake_group;
 
 	g_return_val_if_fail (EMPATHY_IS_CONTACT_LIST_VIEW (view), NULL);
 
@@ -1557,8 +1662,9 @@ empathy_contact_list_view_get_group_menu (EmpathyContactListView *view)
 		return NULL;
 	}
 
-	group = empathy_contact_list_view_get_selected_group (view);
-	if (!group) {
+	group = empathy_contact_list_view_get_selected_group (view, &is_fake_group);
+	if (!group || is_fake_group) {
+		/* We can't alter fake groups */
 		return NULL;
 	}
 
