@@ -75,6 +75,7 @@ typedef struct {
   gboolean enter_create_forward;
   TpAccountManager *account_mgr;
   EmpathyConnectionManagers *connection_mgrs;
+  gint current_page_id;
 
   /* enter or create page */
   GtkWidget *enter_or_create_page;
@@ -147,6 +148,7 @@ build_error_vbox (const gchar *primary_message,
   gtk_label_set_use_markup (GTK_LABEL (w), TRUE);
   gtk_box_pack_start (GTK_BOX (main_vbox), w, FALSE, FALSE, 0);
   gtk_misc_set_alignment (GTK_MISC (w), 0, 0.5);
+  gtk_label_set_line_wrap (GTK_LABEL (w), TRUE);
   gtk_widget_show (w);
 
   return main_vbox;
@@ -275,6 +277,8 @@ account_assistant_account_enabled_cb (GObject *source,
   EmpathyAccountAssistantPriv *priv = GET_PRIV (self);
   const gchar *protocol;
   TpAccount *account = TP_ACCOUNT (source);
+  gint current_idx;
+  gboolean salut_created = FALSE;
 
   tp_account_set_enabled_finish (account, result, &error);
 
@@ -287,13 +291,17 @@ account_assistant_account_enabled_cb (GObject *source,
   protocol = tp_account_get_protocol (account);
   if (!tp_strdiff (protocol, "local-xmpp"))
     {
-      DEBUG ("Salut account has been created; update gconf key");
-
-      empathy_conf_set_bool (empathy_conf_get (),
-          EMPATHY_PREFS_SALUT_ACCOUNT_CREATED,
-          TRUE);
+      salut_created = TRUE;
     }
 
+  empathy_connect_new_account (account, priv->account_mgr);
+
+  current_idx = gtk_assistant_get_current_page (GTK_ASSISTANT (self));
+  if (current_idx == PAGE_SALUT && !salut_created)
+    /* We are on the Salut page and aren't creating the salut account so don't
+     * terminate the assistant. */
+    return;
+
   if (priv->create_enter_resp == RESPONSE_CREATE_STOP)
     g_signal_emit_by_name (self, "close");
   else
@@ -310,22 +318,9 @@ account_assistant_apply_account_cb (GObject *source,
   EmpathyAccountAssistantPriv *priv = GET_PRIV (self);
   EmpathyAccountSettings *settings = EMPATHY_ACCOUNT_SETTINGS (source);
   TpAccount *account;
-  gchar *display_name;
 
   empathy_account_settings_apply_finish (settings, result, &error);
 
-  if (empathy_account_settings_get_display_name (settings) == NULL)
-    {
-      /* set default display name */
-      display_name = empathy_account_widget_get_default_display_name (
-          priv->current_widget_object);
-
-      empathy_account_settings_set_display_name_async (settings,
-          display_name, NULL, NULL);
-
-      g_free (display_name);
-    }
-
   priv->is_creating = FALSE;
 
   if (error != NULL)
@@ -344,7 +339,8 @@ account_assistant_apply_account_cb (GObject *source,
 
 static void
 account_assistant_apply_account_and_finish (EmpathyAccountAssistant *self,
-    EmpathyAccountSettings *settings)
+    EmpathyAccountSettings *settings,
+    gboolean set_display_name)
 {
   EmpathyAccountAssistantPriv *priv = GET_PRIV (self);
 
@@ -353,6 +349,19 @@ account_assistant_apply_account_and_finish (EmpathyAccountAssistant *self,
 
   priv->is_creating = TRUE;
 
+  if (set_display_name)
+    {
+      gchar *display_name;
+
+      display_name = empathy_account_widget_get_default_display_name (
+        priv->current_widget_object);
+
+      empathy_account_settings_set_display_name_async (settings,
+        display_name, NULL, NULL);
+
+      g_free (display_name);
+    }
+
   empathy_account_settings_apply_async (settings,
       account_assistant_apply_account_cb, self);
 }
@@ -379,28 +388,46 @@ account_assistant_protocol_changed_cb (GtkComboBox *chooser,
   char *str;
   GtkWidget *account_widget;
   EmpathyAccountWidget *widget_object = NULL;
-  gboolean is_gtalk;
+  gboolean is_gtalk, is_facebook;
+  const gchar *name;
 
   priv = GET_PRIV (self);
 
   cm = empathy_protocol_chooser_dup_selected (
-      EMPATHY_PROTOCOL_CHOOSER (chooser), &proto, &is_gtalk);
+      EMPATHY_PROTOCOL_CHOOSER (chooser), &proto, &is_gtalk, &is_facebook);
 
   if (cm == NULL || proto == NULL)
     /* we are not ready yet */
     return;
 
   /* Create account */
+  if (is_gtalk)
+    name = "gtalk";
+  else if (is_facebook)
+    name = "facebook";
+  else
+    name = proto->name;
+
   /* To translator: %s is the protocol name */
   str = g_strdup_printf (_("New %s account"),
-      empathy_protocol_name_to_display_name (
-          is_gtalk ? "gtalk" : proto->name));
+      empathy_protocol_name_to_display_name (name));
 
   settings = empathy_account_settings_new (cm->name, proto->name, str);
 
   if (is_gtalk)
-    empathy_account_settings_set_icon_name_async (settings, "im-google-talk",
-      NULL, NULL);
+    {
+      empathy_account_settings_set_icon_name_async (settings, "im-google-talk",
+        NULL, NULL);
+      /* We should not have to set the server but that may cause issue with
+       * buggy router. */
+      empathy_account_settings_set_string (settings, "server",
+          "talk.google.com");
+    }
+  else if (is_facebook)
+    {
+      empathy_account_settings_set_icon_name_async (settings, "im-facebook",
+        NULL, NULL);
+    }
 
   if (priv->first_resp == RESPONSE_CREATE_ACCOUNT)
     empathy_account_settings_set_boolean (settings, "register", TRUE);
@@ -426,6 +453,12 @@ account_assistant_protocol_changed_cb (GtkComboBox *chooser,
   g_signal_connect (priv->current_widget_object, "handle-apply",
       G_CALLBACK (account_assistant_handle_apply_cb), self);
 
+  if (empathy_account_settings_is_valid (settings))
+    {
+      gtk_assistant_set_page_complete (GTK_ASSISTANT (self),
+          priv->enter_or_create_page, TRUE);
+    }
+
   gtk_box_pack_start (GTK_BOX (priv->enter_or_create_page), account_widget,
       FALSE, FALSE, 0);
   gtk_widget_show (account_widget);
@@ -438,6 +471,7 @@ account_assistant_chooser_enter_details_filter_func (
     TpConnectionManager *cm,
     TpConnectionManagerProtocol *protocol,
     gboolean is_gtalk,
+    gboolean is_facebook,
     gpointer user_data)
 {
   if (!tp_strdiff (protocol->name, "local-xmpp") ||
@@ -452,9 +486,10 @@ account_assistant_chooser_create_account_filter_func (
     TpConnectionManager *cm,
     TpConnectionManagerProtocol *protocol,
     gboolean is_gtalk,
+    gboolean is_facebook,
     gpointer user_data)
 {
-  if (is_gtalk)
+  if (is_gtalk || is_facebook)
     return FALSE;
 
   return tp_connection_manager_protocol_can_register (protocol);
@@ -860,14 +895,18 @@ impl_signal_apply (GtkAssistant *assistant)
   if (current_page == PAGE_SALUT)
     {
       if (priv->create_salut_account)
-        account_assistant_apply_account_and_finish (self, priv->salut_settings);
+        /* create_salut_account_settings() already set the display name of the
+         * account so there is no need to set it again. */
+        account_assistant_apply_account_and_finish (self, priv->salut_settings,
+            FALSE);
       return;
     }
-
-  if (current_page >= PAGE_ENTER_CREATE)
-    account_assistant_apply_account_and_finish (self, priv->settings);
-  else if (current_page == PAGE_IMPORT)
-    empathy_import_widget_add_selected_accounts (priv->iw);
+  else if (current_page == PAGE_ENTER_CREATE &&
+      priv->settings != NULL &&
+      empathy_account_settings_is_valid (priv->settings))
+    {
+      account_assistant_apply_account_and_finish (self, priv->settings, TRUE);
+    }
 }
 
 static void
@@ -884,11 +923,20 @@ impl_signal_prepare (GtkAssistant *assistant,
   EmpathyAccountAssistantPriv *priv = GET_PRIV (self);
   gint current_idx;
 
-  current_idx = gtk_assistant_get_current_page (assistant);
-
-  if (current_idx >= PAGE_ENTER_CREATE && current_idx != PAGE_SALUT)
+  /* check from which page we are coming from */
+  switch (priv->current_page_id)
     {
-      if (!priv->enter_create_forward)
+      case PAGE_IMPORT:
+        empathy_import_widget_add_selected_accounts (priv->iw);
+        break;
+    }
+
+  current_idx = gtk_assistant_get_current_page (assistant);
+  priv->current_page_id = current_idx;
+
+  if (current_idx >= PAGE_ENTER_CREATE)
+    {
+      if (!priv->enter_create_forward && current_idx != PAGE_SALUT)
         {
           account_assistant_finish_enter_or_create_page (self,
               priv->first_resp == RESPONSE_ENTER_ACCOUNT ?
@@ -897,7 +945,15 @@ impl_signal_prepare (GtkAssistant *assistant,
       else
         {
           priv->enter_create_forward = FALSE;
-          account_assistant_apply_account_and_finish (self, priv->settings);
+        }
+
+      if (priv->settings != NULL &&
+          empathy_account_settings_is_valid (priv->settings))
+        {
+          account_assistant_apply_account_and_finish (self, priv->settings,
+              TRUE);
+          g_object_unref (priv->settings);
+          priv->settings = NULL;
         }
     }
 }
@@ -1120,9 +1176,9 @@ account_assistant_build_salut_error_page (EmpathyAccountAssistant *self)
 
   markup = g_strdup_printf ("%s (<span style=\"italic\">%s</span>).",
       _("You won't be able to chat with people connected to your local "
-        "network, as telepathy-salut is not installed.\nIf you want to enable "
-        "this feature, please install the telepathy-salut package\nand create "
-        "a People Nearby account from the Accounts dialog "),
+        "network, as telepathy-salut is not installed. If you want to enable "
+        "this feature, please install the telepathy-salut package and create "
+        "a People Nearby account from the Accounts dialog"),
         _("Edit->Accounts"));
 
   vbox = build_error_vbox (_("telepathy-salut not installed"), markup);
@@ -1170,6 +1226,9 @@ empathy_account_assistant_init (EmpathyAccountAssistant *self)
       EmpathyAccountAssistantPriv);
   self->priv = priv;
 
+  gtk_window_set_title (GTK_WINDOW (self),
+      _("Messaging and VoIP Accounts Assistant"));
+
   priv->account_mgr = tp_account_manager_dup ();
 }
 
@@ -1233,6 +1292,12 @@ do_constructed (GObject *object)
       gtk_assistant_set_page_type (assistant, page, GTK_ASSISTANT_PAGE_CONFIRM);
 
       priv->create_salut_account = TRUE;
+
+      if (empathy_account_settings_is_valid (priv->salut_settings))
+        {
+          gtk_assistant_set_page_complete (GTK_ASSISTANT (self),
+              page, TRUE);
+        }
     }
   else
     {
@@ -1261,7 +1326,7 @@ empathy_account_assistant_show (GtkWindow *window,
 
   if (dialog == NULL)
     {
-      dialog =  g_object_new (EMPATHY_TYPE_ACCOUNT_ASSISTANT,
+      dialog = g_object_new (EMPATHY_TYPE_ACCOUNT_ASSISTANT,
           "parent-window", window,
           "connection-managers", connection_mgrs,
           NULL);
