@@ -61,11 +61,6 @@
 #define DEBUG_FLAG EMPATHY_DEBUG_CHAT
 #include <libempathy/empathy-debug.h>
 
-typedef struct {
-	EmpathyChatWindow *window;
-	EmpathyChat *chat;
-} NotificationData;
-
 #define GET_PRIV(obj) EMPATHY_GET_PRIV (obj, EmpathyChatWindow)
 typedef struct {
 	EmpathyChat *current_chat;
@@ -79,7 +74,6 @@ typedef struct {
 	GtkWidget   *dialog;
 	GtkWidget   *notebook;
 	NotifyNotification *notification;
-	NotificationData *notification_data;
 
 	GtkTargetList *contact_targets;
 	GtkTargetList *file_targets;
@@ -333,16 +327,21 @@ chat_window_menu_context_update (EmpathyChatWindowPriv *priv,
 {
 	gboolean first_page;
 	gboolean last_page;
+	gboolean wrap_around;
 	gboolean is_connected;
 	gint     page_num;
 
 	page_num = gtk_notebook_get_current_page (GTK_NOTEBOOK (priv->notebook));
 	first_page = (page_num == 0);
 	last_page = (page_num == (num_pages - 1));
+	g_object_get (gtk_settings_get_default (), "gtk-keynav-wrap-around",
+		      &wrap_around, NULL);
 	is_connected = empathy_chat_get_tp_chat (priv->current_chat) != NULL;
 
-	gtk_action_set_sensitive (priv->menu_tabs_next, TRUE);
-	gtk_action_set_sensitive (priv->menu_tabs_prev, TRUE);
+	gtk_action_set_sensitive (priv->menu_tabs_next, (!last_page ||
+							 wrap_around));
+	gtk_action_set_sensitive (priv->menu_tabs_prev, (!first_page ||
+							 wrap_around));
 	gtk_action_set_sensitive (priv->menu_tabs_detach, num_pages > 1);
 	gtk_action_set_sensitive (priv->menu_tabs_left, !first_page);
 	gtk_action_set_sensitive (priv->menu_tabs_right, !last_page);
@@ -1183,31 +1182,14 @@ chat_window_set_urgency_hint (EmpathyChatWindow *window,
 	gtk_window_set_urgency_hint (GTK_WINDOW (priv->dialog), urgent);
 }
 
-static void
-free_notification_data (NotificationData *data)
-{
-	g_object_unref (data->chat);
-	g_slice_free (NotificationData, data);
-}
-
 static void
 chat_window_notification_closed_cb (NotifyNotification *notify,
-				    NotificationData *cb_data)
+				    EmpathyChatWindow *self)
 {
-	EmpathyNotificationClosedReason reason = 0;
-	EmpathyChatWindowPriv *priv = GET_PRIV (cb_data->window);
-
-#ifdef notify_notification_get_closed_reason
-	reason = notify_notification_get_closed_reason (notify);
-#endif
-	if (reason == EMPATHY_NOTIFICATION_CLOSED_DISMISSED) {
-		empathy_chat_window_present_chat (cb_data->chat);
-	}
+	EmpathyChatWindowPriv *priv = GET_PRIV (self);
 
 	g_object_unref (notify);
 	priv->notification = NULL;
-	free_notification_data (cb_data);
-	priv->notification_data = NULL;
 }
 
 static void
@@ -1242,17 +1224,11 @@ chat_window_show_or_update_notification (EmpathyChatWindow *window,
 		notify_notification_update (priv->notification,
 					    header, escaped, NULL);
 	} else {
-		NotificationData *cb_data = cb_data = g_slice_new0 (NotificationData);
-
-		cb_data->chat = g_object_ref (chat);
-		cb_data->window = window;
-
-		priv->notification_data = cb_data;
 		priv->notification = notify_notification_new (header, escaped, NULL, NULL);
 		notify_notification_set_timeout (priv->notification, NOTIFY_EXPIRES_DEFAULT);
 
 		g_signal_connect (priv->notification, "closed",
-				  G_CALLBACK (chat_window_notification_closed_cb), cb_data);
+				  G_CALLBACK (chat_window_notification_closed_cb), window);
 	}
 
 	pixbuf = empathy_notify_manager_get_pixbuf_for_notification (priv->notify_mgr,
@@ -1757,11 +1733,6 @@ chat_window_finalize (GObject *object)
 		notify_notification_close (priv->notification, NULL);
 		g_object_unref (priv->notification);
 		priv->notification = NULL;
-		if (priv->notification_data != NULL)
-			{
-				free_notification_data (priv->notification_data);
-				priv->notification_data = NULL;
-			}
 	}
 
 	if (priv->contact_targets) {
