@@ -32,6 +32,8 @@
 #include <glib/gi18n.h>
 #include <glib/gprintf.h>
 
+#include <telepathy-glib/interfaces.h>
+
 #include <libempathy/empathy-tp-roomlist.h>
 #include <libempathy/empathy-chatroom.h>
 #include <libempathy/empathy-utils.h>
@@ -131,6 +133,48 @@ static void	new_chatroom_dialog_button_close_error_clicked_cb   (GtkButton
 
 static EmpathyNewChatroomDialog *dialog_p = NULL;
 
+/**
+ * empathy_account_chooser_filter_supports_multichat:
+ * @account: a #TpAccount
+ * @user_data: user data or %NULL
+ *
+ * An #EmpathyAccountChooserFilterFunc that returns accounts that both
+ * support multiuser text chat and are connected.
+ *
+ * Return value: TRUE if @account both supports muc and is connected
+ */
+static gboolean
+empathy_account_chooser_filter_supports_multichat (TpAccount *account,
+						   gpointer   user_data)
+{
+	TpConnection *connection;
+	EmpathyDispatcher *dispatcher;
+	GList *classes;
+
+	if (tp_account_get_connection_status (account, NULL) !=
+	    TP_CONNECTION_STATUS_CONNECTED)
+	return FALSE;
+
+	/* check if CM supports multiuser text chat */
+	connection = tp_account_get_connection (account);
+	if (connection == NULL)
+		return FALSE;
+
+	dispatcher = empathy_dispatcher_dup_singleton ();
+
+	classes = empathy_dispatcher_find_requestable_channel_classes
+		(dispatcher, connection, TP_IFACE_CHANNEL_TYPE_TEXT,
+		TP_HANDLE_TYPE_ROOM, NULL);
+
+	g_object_unref (dispatcher);
+
+	if (classes == NULL)
+		return FALSE;
+
+	g_list_free (classes);
+	return TRUE;
+}
+
 void
 empathy_new_chatroom_dialog_show (GtkWindow *parent)
 {
@@ -202,7 +246,7 @@ empathy_new_chatroom_dialog_show (GtkWindow *parent)
 	/* Account chooser for custom */
 	dialog->account_chooser = empathy_account_chooser_new ();
 	empathy_account_chooser_set_filter (EMPATHY_ACCOUNT_CHOOSER (dialog->account_chooser),
-					    empathy_account_chooser_filter_is_connected,
+					    empathy_account_chooser_filter_supports_multichat,
 					    NULL);
 	gtk_table_attach_defaults (GTK_TABLE (dialog->table_info),
 				   dialog->account_chooser,
@@ -438,6 +482,9 @@ new_chatroom_dialog_account_changed_cb (GtkComboBox             *combobox,
 	EmpathyAccountChooser *account_chooser;
 	gboolean               listing = FALSE;
 	gboolean               expanded = FALSE;
+	TpConnection          *connection;
+	EmpathyDispatcher     *dispatcher;
+	GList                 *classes = NULL;
 
 	if (dialog->room_list) {
 		g_object_unref (dialog->room_list);
@@ -454,13 +501,31 @@ new_chatroom_dialog_account_changed_cb (GtkComboBox             *combobox,
 
 	account_chooser = EMPATHY_ACCOUNT_CHOOSER (dialog->account_chooser);
 	dialog->account = empathy_account_chooser_dup_account (account_chooser);
+	connection = empathy_account_chooser_get_connection (account_chooser);
 	if (dialog->account == NULL)
 		goto out;
 
 	dialog->status_changed_id = g_signal_connect (dialog->account,
 		      "status-changed", G_CALLBACK (account_status_changed_cb), dialog);
 
-	dialog->room_list = empathy_tp_roomlist_new (dialog->account);
+	dispatcher = empathy_dispatcher_dup_singleton ();
+
+	if (connection != NULL) {
+		classes = empathy_dispatcher_find_requestable_channel_classes (dispatcher,
+			connection, TP_IFACE_CHANNEL_TYPE_ROOM_LIST,
+			TP_HANDLE_TYPE_NONE, NULL);
+	}
+
+	if (classes != NULL) {
+		/* Roomlist channels are supported */
+		dialog->room_list = empathy_tp_roomlist_new (dialog->account);
+		g_list_free (classes);
+	}
+	else {
+		dialog->room_list = NULL;
+	}
+
+	g_object_unref (dispatcher);
 
 	if (dialog->room_list) {
 		g_signal_connect (dialog->room_list, "destroy",
@@ -492,6 +557,8 @@ new_chatroom_dialog_account_changed_cb (GtkComboBox             *combobox,
 		}
 	}
 
+	gtk_widget_set_sensitive (dialog->expander_browse, dialog->room_list != NULL);
+
 out:
 	new_chatroom_dialog_update_widgets (dialog);
 }
@@ -534,9 +601,9 @@ new_chatroom_dialog_new_room_cb (EmpathyTpRoomlist        *room_list,
 	selection = gtk_tree_view_get_selection (view);
 	store = GTK_LIST_STORE (dialog->model);
 	members = g_strdup_printf ("%d", empathy_chatroom_get_members_count (chatroom));
-	tooltip = g_strdup_printf (C_("Room/Join's roomlist tooltip. Parameters"
-		"are a channel name, yes/no, yes/no and a number.",
-		"<b>%s</b>\nInvite required: %s\nPassword required: %s\nMembers: %s"),
+	/* Translators: Room/Join's roomlist tooltip. Parameters are a channel name,
+	yes/no, yes/no and a number. */
+	tooltip = g_strdup_printf (_("<b>%s</b>\nInvite required: %s\nPassword required: %s\nMembers: %s"),
 		empathy_chatroom_get_name (chatroom),
 		empathy_chatroom_get_invite_only (chatroom) ? _("Yes") : _("No"),
 		empathy_chatroom_get_need_password (chatroom) ? _("Yes") : _("No"),
