@@ -240,24 +240,12 @@ migrate_config_to_xdg_dir (void)
   g_free (old_dir);
 }
 
-static void
-accounts_application_exited_cb (GPid pid,
-    gint status,
-    gpointer data)
-{
-  if (status)
-    {
-      g_warning ("accounts application exited with status %d: '%s'",
-          status, g_strerror (status));
-    }
-}
-
 static void
 show_accounts_ui (GdkScreen *screen,
     gboolean if_needed)
 {
   empathy_accounts_dialog_show_application (screen,
-      accounts_application_exited_cb, NULL, NULL, if_needed, start_hidden);
+      NULL, if_needed, start_hidden);
 }
 
 static UniqueResponse
@@ -383,8 +371,8 @@ account_manager_ready_cb (GObject *source_object,
       empathy_idle_set_state (idle, TP_CONNECTION_PRESENCE_TYPE_AVAILABLE);
 
   /* Pop up the accounts dialog if it's needed (either when we don't have any
-   * non-salut accounts yet or when we haven't imported mc4 accounts yet */
-  if (!empathy_accounts_has_non_salut_accounts (manager)
+   * accounts yet or when we haven't imported mc4 accounts yet */
+  if (!empathy_accounts_has_accounts (manager)
       || !empathy_import_mc4_has_imported ())
     show_accounts_ui (gdk_screen_get_default (), TRUE);
 
